

library(magrittr)
library(ggplot2)
library(reshape)
library(directlabels)

rm(list=ls())


# Tema
theme_paper<- function()
{
	theme_bw(base_size=26)+
	theme(
		legend.position='none',
		legend.key=element_rect(colour='white'),
		strip.text.y=element_blank(),
		strip.background=element_blank(),
		axis.title.x=element_blank(),
		axis.title.y=element_blank(),
		panel.grid.major=element_line(colour='lightgrey'),
		panel.border=element_rect(colour='black',size=1.5)
	)
}

fold<- 'superficie_N-rho'
nome_x<- 'N'
nome_y<- 'rho'
nome_z<- 'rmse1'
theta<- c(1:4,0.1)
tabella<- NULL

for (subfold in dir(fold))
{
	paste0(fold,'/',subfold) %>% dir -> filen
	filen %>% substr(1,6) %>% {which(.=='piano_')} %>% filen[.] %>% paste0(fold,'/',subfold,'/',.) -> filen
	load(filen)
	piano$.layer<- which(subfold==dir(fold))
	piano$.z<- NA

	for (j in 1:nrow(piano))
	{
		load(paste0(fold,'/',subfold,'/',piano$nomef[j],'.RData'))
		# Filtro p-value
		#out$hpval %>% {which(.>0.05)} %>% min(out$niter) ->pos
		#out$estimate<- out$hestimate[pos,]
		#out$hestimate %<>% extract(1:pos,)
		#out$hgam %<>% extract(1:pos)
		#out$hpval %<>% extract(1:pos)
		#if (out$code==-1) { out$code<- ((out$hpval[pos]>=0.05) & pos<out$niter)-1 }
		#out$niter<- pos
		#out$tempo<- NA
		# Selettore
		p<- (1:length(theta))*2-1
		# Calcoli
		if (nome_z=='ninf_bias') { {out$estimate[p]-theta} %>% abs %>% max -> piano$.z[j] }
		if (nome_z=='n1_bias') { {out$estimate[p]-theta} %>% abs %>% sum -> piano$.z[j] }
		if (nome_z=='ninf_relbias') { {(out$estimate[p]-theta)/theta} %>% abs %>% max -> piano$.z[j] }
		if (nome_z=='n1_relbias') { {(out$estimate[p]-theta)/theta} %>% abs %>% sum -> piano$.z[j] }
		if (nome_z=='tempo') { out$tempo -> piano$.z[j] }
		if (nome_z=='niter') { out$niter -> piano$.z[j] }
		if (nome_z=='code') { out$code -> piano$.z[j] }
		if (nome_z=='gam') { rev(out$hgam)[1] -> piano$.z[j] }
		if (nome_z=='rmse1') { {(out$estimate[p]-theta)[1]} %>% {.^2} -> piano$.z[j] }
		if (nome_z=='rmse2') { {(out$estimate[p]-theta)[2]} %>% {.^2} -> piano$.z[j] }
		if (nome_z=='rmse3') { {(out$estimate[p]-theta)[3]} %>% {.^2} -> piano$.z[j] }
		if (nome_z=='rmse4') { {(out$estimate[p]-theta)[4]} %>% {.^2} -> piano$.z[j] }
		if (nome_z=='rmse5') { {(out$estimate[p]-theta)[5]} %>% {.^2} -> piano$.z[j] }
	}
	
	if (nome_z=='gam') { piano$.z<- piano$.z/abs(piano$.z[1])}
	
	tabella %<>% rbind(piano)
}

tabella %<>% extract(,c(nome_x,nome_y,'.z')) %>% {aggregate(.[,3],by=list(.[,1],.[,2]),FUN=function(x) {sqrt(mean(x))})}
colnames(tabella)<- c(nome_x,nome_y,nome_z)

#tabella[,3] %<>% log
#tabella[,3] %<>% divide_by(60)

pdf(paste0('grafici_paper/Nrho_N',nome_z,'.pdf'))

# Curves wrt N
tabella$gruppo<- as.factor(tabella[,nome_y])
tabella %>% subset(gruppo %in% c(0.01,0.04,0.07,0.1)) %>%
	{ggplot(.,aes_string(x=nome_x,y=nome_z))+geom_line(aes(linetype=gruppo),size=1.5,alpha=0.7)+scale_x_log10()+
		scale_linetype_manual(values=c('solid','42','3212','11'))+scale_y_continuous(limits=c(0.55,0.85))+theme_paper()} ->gr1
print(gr1)

dev.off()

pdf(paste0('grafici_paper/Nrho_rho',nome_z,'.pdf'))

# Curves wrt rho
tabella$gruppo<- as.factor(tabella[,nome_x])
tabella %>% subset(gruppo %in% c(560,5600,56000)) %>%
	{ggplot(.,aes_string(x=nome_y,y=nome_z))+scale_x_reverse()+geom_line(aes(linetype=gruppo),size=1.5,alpha=0.7)+
		scale_linetype_manual(values=c('solid','42','11'))+scale_y_continuous(limits=c(0.55,0.85))+theme_paper()} ->gr1
print(gr1)

dev.off()




box_wrt_method<- function(indbe=1,sigma=1)
{
	load(paste0('distribuzioni/hist',indbe,'.RData'))

	tabella %>% {subset(.,.$G==50 & .$sigma==sigma)} ->df
	levels(df$method)<- c('AGQ (m=1)','AGQ (m=3)','AGQ (m=5)','CE (N=1000)','CE (N=2000)','CE (N=5000)')

	gr<- ggplot(df,aes(x=method,y=stat))+
		geom_boxplot(outlier.size=3)+coord_flip()+
		#scale_y_log10()+
		scale_x_discrete(limits=rev(levels(df$method)))+
		#theme(axis.title=element_blank())
		theme_paper()
	pdf(paste0('grafici_paper/box',indbe,'_',formatC(sigma*100,digits=0,width=3,format='f',flag=0),'_G50.pdf')); print(gr); dev.off()
}



box_wrt_sigma<- function(indbe=1,indmeth=1)
{
	load(paste0('distribuzioni/hist',indbe,'.RData'))

	tabella %>% {subset(.,.$G==50 & .$method==levels(tabella$method)[indmeth])} ->df
	levels(df$method)<- c('AGQ1','AGQ3','AGQ5','CE1000','CE2000','CE5000')
	df$sigma<- paste0('sigma==\"',df$sigma,'\"')

	gr<- ggplot(df,aes(x=sigma,y=stat))+
		geom_boxplot(outlier.size=3)+coord_flip()+
		scale_x_discrete(labels=function(x) parse(text=x))+
		#scale_y_log10()+
		#scale_x_discrete(limits=rev(levels(df$method)))+
		#theme(axis.title=element_blank())
		theme_paper()
	pdf(paste0('grafici_paper/box',indbe,'_',levels(df$method)[indmeth],'_G50.pdf')); print(gr); dev.off()
}



# Boxplot (confronto dei metodi)
for (j in 1:5) { for (h in 1) { box_wrt_method(j,h) }}

# Boxplot (confronto su sigma)
for (j in 3:4) {for (h in 1:5) { box_wrt_sigma(h,j) }}



# Tabella
# tabella %>% reshape(v.names=nome_z,timevar=nome_y,idvar=nome_x,direction='wide') %>% {.[,-1]<-.[,-1]*100; .} %>% round(2) %>% write.table(sep=' & ',eol=' \\\\\n',row.names=FALSE,)



