
library(magrittr)
#library(Matrix)
#library(numDeriv)
library(lme4)
#library(fastGHQuad)
library(maxLik)
#library(SparseGrid)
#library(NominalLogisticBiplot)
library(coda)

rm(list=ls())

set.seed(1861)

source('loglikBin.R')
source('noisyCE.R')

# Piano di simulazione
linux_id<- '489202'
cartella<- 'confronto/xxx'
piano<- expand.grid('sigma'=c(0.01,0.1,0.2,0.5,1),'G'=c(25,50))
piano$nomef<- paste0(linux_id,'_',sprintf('%06d',1:nrow(piano)))
attr(piano,'out.attrs')<- NULL
save(piano,file=paste0(cartella,'/piano_',linux_id,'.RData'))

# Modello
n<- 250
G<- piano$G[1]
k1<- 4
reor<- 1 	# reor: random effects on regressors
k2<- length(reor)
theta<- c(1:k1,piano$sigma[1])
names(theta)<- c(paste0('beta',0:(k1-1)),'sigma')


# Generazione dati
# Data
dati<- data.frame(num=1:n,g=NA)
# Matrix of regressors
{rnorm(n*(k1-1),,1)} %>% matrix(n,k1-1) %>% cbind(1,.) ->X
colnames(X)<- paste0('X',0:(ncol(X)-1))
dati %<>% cbind(X)
# Function extracting beta coefficients from parameters vector
be<- function(par) { par[1:k1] }

# Inversione parametri lognormale
getparlnorm<- function(mu,sigma)
{
	out<- c('logmu'=NA,'logsigma'=NA)
	out[1]<- log(mu/sqrt(1+(sigma/mu)^2))
	out[2]<- sqrt(log(1+(sigma/mu)^2))
	out
}

# "Randomize"
set.seed(NULL)


# Fit the models
for (j in 1:nrow(piano))
{
	cat('Model ',j,':\n',sep='')
	if ((G!=piano$G[j]) | (j==1))
	{
		G<- piano$G[j]
		dati$g<- c(sample(1:G,G),sample(1:G,n-G,TRUE))
		# Matrix "R"
		dati %>% {table(.$n,.$g)} %>% unname ->R
		# Matrix "C"
		matrix(0,k2,k1) %>% {.[,reor]<- diag(1,k2); .} ->C
		# Generator process of random effects
		rugen<- function(x) { rnorm(G*k2,,x[k1+1]) %>% matrix(G,k2) %>% return }
		# 
		MulK<- 10
	}
	# Draw...
	seme<- .Random.seed
	dati$y<- simBin(theta,be,rugen,X,R,C)
	S<- function(x,u) { logcondens(x,be,u,dati$y,X,R,C) }
	#swS<- function(u,x,logscala) { condens(x,be,matrix(u,G,k2),dati$y,X,R,C,logscala) }
	
	# integrazione numerica (n=2)
	#cat('  Gauss-Hermite quadrature (n=2) ...   ')
	#t0<- proc.time()
	#logscala<- S(theta,matrix(0,G,k2))
	#dati$y %>% mean %>% {.*log(.)+(1-.)*log(1-.)} %>% multiply_by(n) ->logscala
	#glm(y~.-1-g-num,data=dati,family='binomial') %>% {c(coef(.),1)} ->incipit
	#{function(x) { ghintegral(swS,dims=G*k2,muHat=0,sigmaHat=rev(x)[1],nnodes=2,x=x,logscala=logscala) }} %>%
	#	maxLik(start=incipit,method='NM',constraints=list(ineqA=matrix(c(rep(0,k1),1),1),ineqB=0),iterlim=10000) ->gh2
	#proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->gh2$tempo
	#save(gh2,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	#cat(' [  ok  ]\n')
	
	# integrazione numerica AGQ (n=1, Laplace)
	cat('  AG quadrature (n=1) ...              ')
	t0<- proc.time()
	glmer(cbind(y,1-y)~0+X0+X1+X2+X3+(0+X0|g),dati,family=binomial,nAGQ=1) ->modAGQ1
	AGQ1<- list(estimate=c(modAGQ1@beta,modAGQ1@theta),code=modAGQ1@optinfo$conv$opt,niter=NA)
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->AGQ1$tempo
	save(AGQ1,modAGQ1,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n')
	
	# integrazione numerica AGQ (n=3)
	cat('  AG quadrature (n=3) ...              ')
	t0<- proc.time()
	glmer(cbind(y,1-y)~0+X0+X1+X2+X3+(0+X0|g),dati,family=binomial,nAGQ=3) ->modAGQ3
	AGQ3<- list(estimate=c(modAGQ3@beta,modAGQ3@theta),code=modAGQ3@optinfo$conv$opt,niter=NA)
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->AGQ3$tempo
	save(AGQ1,modAGQ1,AGQ3,modAGQ3,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n')
	
	# integrazione numerica AGQ (n=5)
	cat('  AG quadrature (n=5) ...              ')
	t0<- proc.time()
	glmer(cbind(y,1-y)~0+X0+X1+X2+X3+(0+X0|g),dati,family=binomial,nAGQ=5) ->modAGQ5
	AGQ5<- list(estimate=c(modAGQ5@beta,modAGQ5@theta),code=modAGQ5@optinfo$conv$opt,niter=NA)
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->AGQ5$tempo
	save(AGQ1,modAGQ1,AGQ3,modAGQ3,AGQ5,modAGQ5,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n')
	
	# entropia relativa (N=1000)
	cat('  Cross-entropy (N=1000) ...           ')
	t0<- proc.time()
	glm(y~.-1-g-num,data=dati,family='binomial') %>% summary ->incipit
	incipit %<>% {rbind(coef(.)[,1],MulK*coef(.)[,2])} %>% {cbind(.,getparlnorm(incipit$deviance/incipit$df.residual,MulK*incipit$deviance/incipit$df.residual))} %>% as.vector
	ce1000<- noisyCE(S,rugen,v0=incipit,domain=c(rep(0,k1),1),
		N=1000,rho=0.04,asmooth=1,bsmooth=0.975,qsmooth=10,pval=0.05,buff=20,vbuff=0.5,pgeweke=c(0.3,0.4))
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->ce1000$tempo
	save(AGQ1,modAGQ1,AGQ3,modAGQ3,AGQ5,modAGQ5,ce1000,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n')
	
	# entropia relativa (N=2000)
	cat('  Cross-entropy (N=2000) ...           ')
	t0<- proc.time()
	glm(y~.-1-g-num,data=dati,family='binomial') %>% summary ->incipit
	incipit %<>% {rbind(coef(.)[,1],MulK*coef(.)[,2])} %>% {cbind(.,getparlnorm(incipit$deviance/incipit$df.residual,MulK*incipit$deviance/incipit$df.residual))} %>% as.vector
	ce2000<- noisyCE(S,rugen,v0=incipit,domain=c(rep(0,k1),1),
		N=2000,rho=0.04,asmooth=1,bsmooth=0.975,qsmooth=10,pval=0.05,buff=20,vbuff=0.5,pgeweke=c(0.3,0.4))
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->ce2000$tempo
	save(AGQ1,modAGQ1,AGQ3,modAGQ3,AGQ5,modAGQ5,ce1000,ce2000,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n')
	
	# entropia relativa (N=5000)
	cat('  Cross-entropy (N=5000) ...           ')
	t0<- proc.time()
	glm(y~.-1-g-num,data=dati,family='binomial') %>% summary ->incipit
	incipit %<>% {rbind(coef(.)[,1],MulK*coef(.)[,2])} %>% {cbind(.,getparlnorm(incipit$deviance/incipit$df.residual,MulK*incipit$deviance/incipit$df.residual))} %>% as.vector
	ce5000<- noisyCE(S,rugen,v0=incipit,domain=c(rep(0,k1),1),
		N=5000,rho=0.04,asmooth=1,bsmooth=0.975,qsmooth=10,pval=0.05,buff=20,vbuff=0.5,pgeweke=c(0.3,0.4))
	proc.time() %>% subtract(t0) %>% extract(3) %>% unname ->ce5000$tempo
	save(AGQ1,modAGQ1,AGQ3,modAGQ3,AGQ5,modAGQ5,ce1000,ce2000,ce5000,dati,seme,file=paste0(cartella,'/',piano$nomef[j],'.RData'))
	cat(' [  ok  ]\n\n')
}


